@@ -11,25 +11,41 @@ class method(object):
     def __init__(self, remote_func):
         self.remote_func = remote_func
         self.instance = None
-    def __call__(self, args):
+    def __call__(self, *args, **kwargs):
         if self.instance == None: raise exception.MethodException("Method instance not set before being called")
-        return self.remote_func(self.instance, *args)
+        return self.remote_func(self.instance, *args, **kwargs)
 
 class SocketObject(threading.Thread):
     remote_functions = []
-    def __init__(self, port, host = '', debug = False):
+    def __init__(self, port, host = '', debug = False, timeout = 1, reuseaddr = True):
         threading.Thread.__init__(self)
         self.lock = threading.Lock()
-        self.log = logging.Logger(debug)
+        self._log = logging.Logger(debug)
         self.running = False
         self.registered = False
         self.port = port
         self.host = host
-        socket.setdefaulttimeout(1)
+        self.reuseaddr = reuseaddr
+        socket.setdefaulttimeout(timeout)
         self.responders = []
-        self.jbus_methods = {member[0] : member[1] for member in inspect.getmembers(self.__class__) if type(member[1]) is method}
-        for jbus_method in self.jbus_methods.values():
-            jbus_method.instance = self
+        self.jrpc_objects = {member[0] : member[1] for member in inspect.getmembers(self.__class__) if type(member[1]) is method}
+        for jrpc_method in self.jrpc_objects.values():
+            jrpc_method.instance = self
+
+    def __setattr__(self, name, value):
+        if isinstance(value, SocketObject):
+            self.jrpc_objects[name] = value
+        threading.Thread.__setattr__(self, name, value)
+
+    def get_method(self, path):
+        if path[0] not in self.jrpc_objects:
+            return None
+        output = self.jrpc_objects[path[0]]
+        if len(path) == 1:
+            if not isinstance(output, method):
+                return None
+            return output
+        return output.get_method(path[1:])
 
     def run_wait(self):
         try:
@@ -43,32 +59,38 @@ class SocketObject(threading.Thread):
 
     def pre_run(self):
         self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        if self.reuseaddr:
+            self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.s.bind((self.host, self.port))
         self.port = self.s.getsockname()[1]              
         self.s.listen(1)
-        self.log.info("Service listening on port {0}".format(self.port))
+        self._log.info("Service listening on port {0}".format(self.port))
 
     def run(self):
         while self.running:
             try:
                 conn, addr = self.s.accept()
-                self.log.info("Got connection from {0}".format(addr))
-                responder = ServiceResponder(self, conn, addr, self.log)
+                self._log.info("Got connection from {0}".format(addr))
+                responder = ServiceResponder(self, conn, addr, self._log)
                 responder.start()
                 self.responders.append(responder)
             except socket.timeout:
                 continue
             except Exception as e:
-                self.log.error("An error occured: {0}".format(e))
+                self._log.error("An error occured: {0}".format(e))
                 self.close()
             
     def close(self):
         for responder in self.responders:
             responder.close()
+        for name in self.jrpc_objects:
+            service = self.jrpc_objects[name]
+            if isinstance(service, SocketObject):
+                service.close()
         if not self.running: return
         self.running = False
         if self.s != None:
-            self.log.info("Closing socket")
+            self._log.info("Closing socket")
             self.s.close()
             del self.s
 
@@ -78,7 +100,7 @@ class ServiceResponder(threading.Thread):
         self.service_obj = service_obj
         self.socket = socket
         self.addr = addr
-        self.log = log
+        self._log = log
         self.running = True
     def run(self):
         recvd = ""
@@ -87,13 +109,14 @@ class ServiceResponder(threading.Thread):
                 msg = message.deserialize(self.socket)
                 if type(msg) is message.Request:
                     response = message.Response(msg.id)
-                    if msg.method in self.service_obj.jbus_methods:
+                    method_target = self.service_obj.get_method(msg.method.split('.'))
+                    if method_target != None:
                         self.service_obj.lock.acquire()
                         try:
-                            response.result = self.service_obj.jbus_methods[msg.method](msg.params)
-                            self.log.info("{0} called \"{1}\" returning {2}".format(self.addr, msg.method, json.dumps(response.result)))
+                            response.result = method_target(*msg.params[0], **msg.params[1])
+                            self._log.info("{0} called \"{1}\" returning {2}".format(self.addr, msg.method, json.dumps(response.result)))
                         except Exception as e:
-                            self.log.info("An exception occured while calling {0}: {1}".format(msg.method, e))
+                            self._log.info("An exception occured while calling {0}: {1}".format(msg.method, e))
                             response.error = exception.exception_to_error(e)
                         finally:
                             self.service_obj.lock.release()
@@ -102,11 +125,11 @@ class ServiceResponder(threading.Thread):
                     
                     response.serialize(self.socket)
                 else:
-                    self.log.info("Got a message of uknown type")
+                    self._log.info("Got a message of uknown type")
             except socket.timeout:
                 continue
             except Exception as e:
-                self.log.info("Client socket exception: {0}".format(e))
+                self._log.info("Client socket exception: {0}".format(e))
                 break
         self.close()
 
@@ -114,7 +137,7 @@ class ServiceResponder(threading.Thread):
         if not self.running: return
         self.running = False
         if self.socket != None:
-            self.log.info("Client disconnected {0}".format(self.addr))
+            self._log.info("Client disconnected {0}".format(self.addr))
             self.socket.close()
             del self.socket
 
@@ -126,12 +149,14 @@ class CallBack(object):
     def __init__(self, procedure_name, proxy):
         self.proxy = proxy
         self.procedure_name = procedure_name
-    def __call__(self, *args):
-        return self.proxy.rpc(self.procedure_name, args)
+    def __getattr__(self, name):
+        return CallBack(self.procedure_name + "." + name, self.proxy)
+    def __call__(self, *args, **kwargs):
+        return self.proxy.rpc(self.procedure_name, args, kwargs)
 
 class SocketProxy(object):
-    def __init__(self, port, host = 'localhost', socktype = socket.SOCK_STREAM):
-        socket.setdefaulttimeout(1)
+    def __init__(self, port, host = 'localhost', socktype = socket.SOCK_STREAM, timeout = 1):
+        socket.setdefaulttimeout(timeout)
         self.socket = None
         self.next_id = 1
         self.lock = threading.Lock()
@@ -144,12 +169,12 @@ class SocketProxy(object):
             if e.args[0] != 111:
                 raise exception.JRPCError("An error occured", e)
 
-    def rpc(self, remote_procedure, args):
+    def rpc(self, remote_procedure, args, kwargs):
         self.lock.acquire()
         try:
             msg = message.Request(self.next_id, remote_procedure)
             self.next_id += 1
-            msg.params = args
+            msg.params = [args, kwargs]
 
             # Attempt sending and connection if neccessary
             try:
@@ -177,7 +202,7 @@ class SocketProxy(object):
             self.lock.release()
 
     def close(self):
-        if self.socket != None:
+        if "socket" in self.__dict__ and self.socket != None:
             self.socket.close()
         self.socket = None
         
